@@ -22,7 +22,7 @@
 # mkbackup is released under the GNU GPL license
 #
 
-import sys, os, subprocess, time, smtplib, calendar, re, socket, urllib2, logging, logging.handlers
+import sys, os, subprocess, time, smtplib, calendar, re, socket, urllib2, logging, logging.handlers, codecs
 
 from optparse import OptionParser
 import ConfigParser
@@ -37,9 +37,11 @@ from datetime import datetime, timedelta
 
 import cron
 
-__version__='0.1.4'
+__version__='0.1.5'
 last_version_url='http://www.magikmon.com/download/mkbackup_last.txt'
 
+windows_codepage='cp1252'
+
 atom=r"[a-zA-Z0-9_#\$&'*+/=?\^`{}~|\-]+"
 dot_atom=atom  +  r"(?:\."  +  atom  +  ")*"
 quoted=r'"(?:\\[^\r\n]|[^\\"])*"'
@@ -94,17 +96,22 @@ def check_mail_config(job, errors):
     new_errors={}
 #    smtp_host=job.setdefault('smtp_host', '127.0.0.1')
 #    smtp_port=job.setdefault('smtp_port', '25')
-    smtp_host=job['smtp_host']
+
+    try:
+        job['smtp_host']=smtp_host=job['smtp_host'].encode(windows_codepage)
+    except UnicodeEncodeError:
+        new_errors['smtp_host']='invalid hostname or ip address'
+    else:
+        if not valid_ipRE.match(smtp_host):
+            if not valid_hostnameRE.match(smtp_host):
+                new_errors['smtp_host']='invalid hostname or ip address'
+            else:
+                try:
+                    ip=socket.gethostbyname(smtp_host)
+                except socket.gaierror:
+                    new_errors['smtp_host']='cannot resolve address'
+
     smtp_port=job['smtp_port']
-    
-    if not valid_ipRE.match(smtp_host):
-        if not valid_hostnameRE.match(smtp_host):
-            new_errors['smtp_host']='invalid hostname or ip address'
-        else:
-            try:
-                ip=socket.gethostbyname(smtp_host)
-            except socket.gaierror:
-                new_errors['smtp_host']='cannot resolve address'
     try:
         port=int(smtp_port)
     except Exception:
@@ -112,6 +119,8 @@ def check_mail_config(job, errors):
     else:
         if not (0<port and port<65535):
             new_errors['smtp_port']='must be an integer between 1 and 65535'
+        else:
+            job['smtp_port']=port
         
     sender=job.get('sender', None)
     if not sender:
@@ -148,20 +157,21 @@ def sendmail(sender, recipients, subject, text, attachements, smtp_host, smtp_po
     msg['Date'] = formatdate(localtime=True)
     msg['Subject'] = subject
 
-#        text=u'\u043a\u0438\u0440\u0438\u043b\u0438\u0446\u0430'
-#        core=MIMEText(text.encode('US-ASCII'), 'plain', 'US-ASCII')
-    core=MIMEText(text, 'plain')
+    core=MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
     msg.attach(core)
     
-    for filename, target, data in attachements:
+    for filename, target, data, maintype, subtype, charset in attachements:
         
         if not data and target and os.path.exists(target):
             data=open(target, 'rb').read()
 
         if data:
             maintype, subtype = 'application', 'octet-stream'
-            attachement=MIMEBase(maintype, subtype)
-            attachement.set_payload(data)
+            if maintype=='text':
+                attachement=MIMEText(data, subtype, charset)
+            else:
+                attachement=MIMEBase(maintype, subtype)
+                attachement.set_payload(data) #, charset) #dont use the charset, here !
             email.Encoders.encode_base64(attachement)
     
         attachement.add_header('Content-Disposition', 'attachment', filename=filename)
@@ -172,6 +182,10 @@ def sendmail(sender, recipients, subject, text, attachements, smtp_host, smtp_po
     smtp=smtplib.SMTP(smtp_host, smtp_port)
     ret=smtp.sendmail(sender, recipients, body)
 
+def convert_args_into_windows_codepage(args):
+    return map(lambda x: x.encode(windows_codepage), args)
+
+
 # ===========================================================================
 class NtBackup:
     # http://support.microsoft.com/kb/814583
@@ -201,6 +215,8 @@ class NtBackup:
 
         errors, warning={}, {}
 
+        up2date, up2date_msg=job['up2date'], job['up2date_msg']
+        
         #
         # check job config
         #
@@ -243,7 +259,11 @@ class NtBackup:
 
         mail_config=not check_mail_config(job, errors)
 
-        msg_body=''
+        
+        msg_body=u''
+        if up2date=='no':
+            msg_body+='\n%s\n' %up2date_msg
+            
         if errors:
             log.error('Error in section: %s', job['name'])
             for k, v in errors.iteritems():
@@ -266,7 +286,7 @@ class NtBackup:
         
         now=datetime.now()
         if command in ('check', 'checkmail'):
-            msg_body+='Destinations day by day:\n'
+            msg_body+='Destinations day by day:\n\n'
             log.info('Destinations day by day:')
             for i in range(destination.max_weekdivisor*7):
                 today=now+timedelta(days=i)
@@ -286,6 +306,7 @@ class NtBackup:
                     log.warning('        directory not found: %s', os.path.dirname(target))
 
         typ, target=destination.match(now)
+        target_dir=os.path.dirname(target)
         
         if typ=='none':
             cmdline='no backup today'
@@ -304,9 +325,8 @@ class NtBackup:
             cmdline=' '.join(args)
             log.info('cmdline=%s', cmdline)
 
-        msg_body+='\n%s\n' % cmdline
-        
         if command=='checkmail':
+            msg_body+='\nCommand line:\n\n%s\n' % cmdline
             subject='MKBACKUP: CONFIG OK %s ' % (job['name'], )
             sendmail(job['sender'], job['recipients'], subject, msg_body, [], job['smtp_host'], job['smtp_port'])
             
@@ -316,9 +336,10 @@ class NtBackup:
         if typ=='none' or typ==None:
             log.info('bye')
             return
-           
+        
+        pargs=convert_args_into_windows_codepage(args)
         start=int(time.time())
-        process=subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        process=subprocess.Popen(pargs, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         std_out, std_err=process.communicate()
         end=int(time.time())
         
@@ -332,60 +353,78 @@ class NtBackup:
         log.info('logfile=%s', logfile)
         
         ev_status, ev_out=ReadEvLog(NtBackup.ev_logtype, NtBackup.ev_source, start)
-
-        status='version=%s\n' % __version__
-        status='returncode=%r\n' % process.returncode
-        status+='start_epoch=%d\n' % start
-        status+='end_epoch=%d\n' % end
-        status+='start=%s\n' % time.ctime(start)
-        status+='end=%s\n' % time.ctime(end)
-        status+='cmdline=%s\n' % cmdline
-        status+='status=%s\n' % ev_status
         
+        status=u''
+        status+='status=%s\r\n' % ev_status
+        status+='version=%s\r\n' % __version__
+        status+='returncode=%r\r\n' % process.returncode
+        status+='start=%s\r\n' % time.ctime(start)
+        status+='end=%s\r\n' % time.ctime(end)
+        
+        status+='selection=%s\r\n' % job.get('selection')
+        status+='target=%s\r\n' % target
         stat=os.stat(target)
-        status+='target_size=%d\n' % stat.st_size
-        status+='target_mtime_epoch=%d\n' % stat.st_mtime
-        status+='target_mtime=%s\n' % time.ctime(stat.st_mtime)
+        status+='target_size=%d\r\n' % stat.st_size
+        status+='target_mtime_epoch=%d\r\n' % stat.st_mtime
+        status+='target_mtime=%s\r\n' % time.ctime(stat.st_mtime)
         
         #import ctypes
         #free_bytes = ctypes.c_ulonglong(0)
-        #ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(os.path.dirname(target)), None, None, ctypes.pointer(free_bytes))
+        #ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(target_dir), None, None, ctypes.pointer(free_bytes))
         #print free_bytes.value
 
         import win32file
-        free_bytes, total_bytes, total_free_bytes=win32file.GetDiskFreeSpaceEx(os.path.dirname(target))
-        status+='target_free_space=%d\n' % total_free_bytes
+        free_bytes, total_bytes, total_free_bytes=win32file.GetDiskFreeSpaceEx(target_dir)
+        status+='target_free_space=%d\r\n' % total_free_bytes
+
+        status+='start_epoch=%d\r\n' % start
+        status+='end_epoch=%d\r\n' % end
+        status+='cmdline=%s\r\n' % cmdline
+
 
         for line in status.split('\n'):
             if line:
                 log.info('    %s', line)
         
-        # HKEY_CURRENT_USER\Software\Microsoft\Ntbackup\Log Files
-        
         #
         # dir of target directory
         #
-        args=[ os.environ['comspec'], '/c', 'dir', os.path.dirname(target) ]
-        process=subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        dir_out, err=process.communicate()
-        
-        attachements=[ ('log.txt', logfile, None),
-                       ('dir.txt', None, dir_out),
-                       ('evlog.txt', None, ev_out),
-                       ('config.ini', job['config'], None),
-                       ('status.txt', None, status),
+        if True:
+            dir_out=u'  epoch   |          time          |      size     |    filename\r\n'
+            for filename in os.listdir(target_dir):
+                full_filename=os.path.join(target_dir, filename)
+                stat=os.stat(full_filename)
+                if os.path.isdir(full_filename):
+                    size='<DIR>'
+                else:
+                    size='%15d' % stat.st_size 
+                dir_out+='%d %s %15s %s\r\n' % (stat.st_mtime, time.ctime(stat.st_mtime), size, filename)
+                if last<current:
+                    logfile, last=full_filename, current
+        else: # OLD code, but use CP850 in my XP FR_en
+            args=[ os.environ['comspec'], '/u', '/c', 'dir', target_dir ]
+            pargs=convert_args_into_windows_codepage(args)
+            process=subprocess.Popen(pargs, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            dir_out, err=process.communicate()
+
+        attachements=[ # the type, subtype and coding is not used
+                       ('log.txt', logfile, None, 'text', 'plain', 'utf-16'),
+                       ('dir.txt', None, dir_out.encode('utf-8'), 'text', 'plain', 'utf-8'),
+                       ('evlog.txt', None, ev_out.encode('utf-8'), 'text', 'plain', 'utf-8'), 
+                       ('config.ini', job['config'], None, 'text', 'plain', windows_codepage),
+                       ('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'),
                     ]
         if job['selection'].startswith('@'):
-            attachements.append(('selection.bks', job['selection'][1:], None))
+            attachements.append(('selection.bks', job['selection'][1:], None, 'text', 'plain', 'utf-16'))
         if std_out:
-            attachements.append(('stdout.txt', None, std_out))
+            attachements.append(('stdout.txt', None, std_out, 'text', 'plain', 'cp850'))
         if std_err:
-            attachements.append(('stderr.txt', None, std_err))
+            attachements.append(('stderr.txt', None, std_err, 'text', 'plain', 'cp850'))
         
+        msg_body+='\n'+status
         subject='MKBACKUP BACKUP %s %s' % (ev_status, job['name'])
-        sendmail(job['sender'], job['recipients'], subject, cmdline, attachements, job['smtp_host'], job['smtp_port'])
+        sendmail(job['sender'], job['recipients'], subject, msg_body, attachements, job['smtp_host'], job['smtp_port'])
         
-
 archivers=dict((x.name, x) for x in (NtBackup, ))
 
 
@@ -413,18 +452,17 @@ def FormatEv(ev_obj, logtype):
     level=str(ev_obj.EventType )
     record=str(ev_obj.RecordNumber)
     evt_id=str(winerror.HRESULT_CODE(ev_obj.EventID))
-    evt_type=str(evt_dict[ev_obj.EventType])
-    msg=str(win32evtlogutil.SafeFormatMessage(ev_obj, logtype))
-    msg='======== %d %s %s =====\n%s' % ( ev_obj.EventID, evt_type, ev_obj.TimeGenerated, msg)
-    #print ev_obj.EventID, evt_type, int(ev_obj.TimeGenerated), level, msg
+    evt_type=evt_dict.get(ev_obj.EventType, 'UNK')
+    msg=win32evtlogutil.SafeFormatMessage(ev_obj, logtype)
+    epoch=int(ev_obj.TimeGenerated)
+    msg=u'======== eventid=%d eventtype=%s epoch=%d time="%s" =====\r\n%s' % ( ev_obj.EventID, evt_type, epoch, time.ctime(epoch), msg)
+    #print ev_obj.EventID, evt_type, int(ev_obj.TimeGenerated), level, msg.encode('UTF-8')
     return msg 
 
 def ReadEvLog(logtype, source, start, end=None):
     flags = win32evtlog.EVENTLOG_BACKWARDS_READ|win32evtlog.EVENTLOG_SEQUENTIAL_READ
-    hand=win32evtlog.OpenEventLog('', logtype) # '' for localhost
-    cont='first'
-    output=''
-    status='ERR'
+    hand=win32evtlog.OpenEventLog(None, logtype) # None for localhost
+    cont, output, status='first', u'', 'ERR'
     while cont:
         events=win32evtlog.ReadEventLog(hand,flags,0)
         for ev_obj in events:
@@ -440,12 +478,15 @@ def ReadEvLog(logtype, source, start, end=None):
                 if ev_obj.EventID==8019 and ev_obj.EventType==win32con.EVENTLOG_INFORMATION_TYPE:
                     status='OK'
                 
-            output=output+'\n'+FormatEv(ev_obj, logtype)
+            if output:
+                output=FormatEv(ev_obj, logtype)+u'\r\n'+output
+            else:
+            		output=FormatEv(ev_obj, logtype)
+
         cont=cont and events
     win32evtlog.CloseEventLog(hand)
     return status, output
 
-
 #======================================================================
 #
 # Destinations
@@ -531,7 +572,9 @@ class Destinations:
             
             for exp in re.findall('\{([^}]*)\}',target):
                 target=target.replace('${%s}' % (exp,), str(eval(exp, globals, {})))
+            target=target.encode('utf-8')
             target=day.strftime(target)
+            target=target.decode('utf-8')
 
             return typ, target
             
@@ -601,18 +644,20 @@ config_default=dict(verify='no',
 
 config=ConfigParser.RawConfigParser(config_default)
 try:
-    config.readfp(open(cmd_options.config))
+    config.readfp(codecs.open(cmd_options.config, 'r', windows_codepage))
 except IOError:
     log.exception('error reading configuration file %s', cmd_options.config)
     sys.exit(1)
 
+up2date, up2date_msg='unk', ''
 check_for_update=config.get('DEFAULT', 'update_check')
 if boolean.get(check_for_update.lower(), None)!=False:
-    up2date, msg=update_check()
+    up2date, up2date_msg=update_check()
     if up2date=='no':
         log.warning('!'*60)
-        for line in msg:
-            log.warning(line)
+        for line in up2date_msg.split('\n'):
+            if line:
+                log.warning(line)
         log.warning('!'*60)
 
 err=0
@@ -621,11 +666,15 @@ for job_name in job_list:
         program=config.get(job_name, 'program')
     except ConfigParser.NoOptionError, e:
         log.error('no "program" set for job "%s"',job_name)
-        err+=1   
+        err+=1
+    except ConfigParser.NoSectionError, e:
+        log.error('job not found: %s',job_name)
+        err+=1
     else:
         if not program in archivers:
             log.error('program "%s" unknow in job "%s"', program, job_name)
-            err+=1   
+            err+=1
+
 if err>0:
     sys.exit(3)
 
@@ -637,6 +686,8 @@ for job_name in job_list:
     job=dict(config.items(job_name))
     job['name']=job_name
     job['config']=cmd_options.config
+    job['up2date']=up2date
+    job['up2date_msg']=up2date_msg
 
     log.info('start command=%s job=%s archiver=%s', command, job_name, program)
     try:
@@ -672,6 +723,45 @@ for job_name in job_list:
 
 
 
+# http://www.scalabium.com/faq/dct0082.htm
+# To detect the code page of Windows operation system you must call the GetACP function from Windows API.
+# If you needs to read the code page of "DOS" sessions, you must call the GetOEMCP function from Windows API. This function will return the value:
+# http://msdn.microsoft.com/en-us/library/dd318070%28VS.85%29.aspx
+
+#
+
 
 # windows unicode file http://www.helpware.net/FAR/help/Unicode2.htm
 # Universal Encoding Detector http://chardet.feedparser.org/docs/supported-encodings.html
+
+
+# wbadmin
+#
+# How to use wbadmin to backup in Windows Vista 2008
+# http://www.howtonetworking.com/Windows/wbadmin1.htm
+#
+# Backup Version and Space Management in Windows Server Backup
+# https://blogs.technet.com/filecab/archive/2009/06/22/backup-version-and-space-management-in-windows-server-backup.aspx
+#
+# Customizing Windows Server Backup Schedule
+# http://blogs.technet.com/filecab/archive/2009/04/13/customizing-windows-server-backup-schedule.aspx
+#
+# %windir%\logs\windowsserverbackup
+#
+# Output of command "Vshadow -wm2" ran on an elevated command prompt
+#
+# Deleting System State Backup:
+# wbadmin delete systemstatebackup -deleteOldest
+# wbadmin delete systemstatebackup -keepversions:10   
+# The above command will keep the latest 10 versions and delete the rest all the system state backups.
+#
+# Deleting other backups:  Backup application stores multiple backup versions in the VSS shadow copies. Hence, older backup version can be deleted by deleting older shadow copy. Commands to list and delete VSS shadow copies are below. They need to be run in an elevated command window.
+# vssadmin list shadows /for=x: ? for listing the snapshots on x: where x: is the backup location
+# vssadmin delete shadows /for=x:  /oldest ? for delete the oldest shadow copy. It can be called multiple times in case there is need to delete multiple older backups.
+# Note: wbadmin get versions or backup UI would still report deleted backups until next backup runs. At end of each backup ? non-existent backups are removed from the backup catalog.
+#
+# Win2k8: the backups are stored inside the <drive_letter>:\WindowsImageBackup\<machineName>\SystemStateBackup
+# Win2k8 R2: the backups are stored inside the <drive_letter>:\WindowsImageBackup\<machineName>\BackupSet<..>\ path
+
+
+ 
